/****************************************************************************

         #####   #####  #     #       #  #####   #####    ###
        #     # #     # ##    #      #  #     # #     #    #
        #       #       # #   #     #   #       #          #
        #       #       #  #  #    #    #        #####     #
        #       #       #   # #   #     #             #    #
        #     # #     # #    ##  #      #     # #     #    #
         #####   #####  #     # #        #####   #####    ###

****************************************************************************/
/*
* @PROJECT  CCN/CSI
* @SYSTEM   APACHE
* @PACKAGE  COMMON
*
* @FILE     function.c
*
* @ABSTRACT
* Apache common functions
*
* This file contains:
* The common functions used by the Apache handlers
*
* @INCLUDE

#include <stdarg.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "gcn_upcon.h"
#include "gcn_uptyp.h"
#include "cdx_upcon.h"
#include "cdx_uptyp.h"
#include "cdx_upext.h"
#include "osx_upcon.h"
#include "osx_upext.h"
#include "qos_prcon.h"

#include "com_upcon.h"
#include "com_upext.h"
#include "com_prmes.h"
#include "com_prtyp.h"
#include "com_prmac.h"

extern CSICHAR *szAction;
extern CSICHAR48 szWebDomain;
extern CSIBOOLEAN OSInit;
extern CSIBOOLEAN CDXBind;


CSIVOID to_upper(CSICHAR *szString)
{
    while ('\0' != szString[0])
    {
        szString[0] = toupper(szString[0]);
        szString++;
    }
}


CSIVOID to_lower(CSICHAR *szString)
{
    while ('\0' != szString[0])
    {
        szString[0] = tolower(szString[0]);
        szString++;
    }
}


CSICHAR *PrintMode(CSILONG lMode)
{
    switch (lMode)
    {
    case CDX_C_PRODUCTION :
        return CDX_C_PRODUCTION_STR;
    case CDX_C_TEST :
        return CDX_C_TEST_STR;
    case CDX_C_BACKUP :
        return CDX_C_BACKUP_STR;
    case CDX_C_MAINTENANCE :
        return CDX_C_MAINTENANCE_STR;
    }
     
    return "Wrong value";
}


CSICHAR *PrintReturnCode(CSILONG lReturnCode)
{
    if (CSISUCCESS == lReturnCode)
        return COM_SUCCESS_STR;      
    return COM_FAILURE_STR;   
}                   
                    

static CSICHAR *PrintOperationalStatus(CSILONG lOpStatus)
{                              
    switch (lOpStatus)         
    {                
    case CDX_C_AVAILABLE :
        return CDX_C_AVAILABLE_STR;
    case CDX_C_UNAVAILABLE :   
        return CDX_C_UNAVAILABLE_STR;
    default :                  
        return CDX_C_WRONGVALUE_STR;
    }                          
}                              
    

static CSICHAR *PrintCodePage(CSILONG lCode)
{                              
    switch (lCode)             
    {            
    case CDX_C_ISO88591 :
        return CDX_C_ISO88591_STR;
    case CDX_C_IBM037 :        
        return CDX_C_IBM037_STR;
    default :                  
        return CDX_C_WRONGVALUE_STR;
    }                          
}                              
    

static CSICHAR *PrintAdminRole(CSILONG lRole)
{                              
    switch (lRole)             
    {            
    case CDX_C_CCMASTER :
        return CDX_C_CCMASTER_STR;
    case CDX_C_LCMASTER :      
        return CDX_C_LCMASTER_STR;
    case CDX_C_ATT_NOT_DEFINED :
        return "";             
    default :                  
        return CDX_C_WRONGVALUE_STR;
    }                          
}                              
    

static CSICHAR *PrintAdminRight(CSILONG lRight)
{                              
    switch (lRight)            
    {             
    case CDX_C_READWRITE :
        return CDX_C_READWRITE_STR;
    case CDX_C_READ :          
        return CDX_C_READ_STR; 
    case CDX_C_LIST :        
        return CDX_C_LIST_STR;
    case CDX_C_MGMT :        
        return CDX_C_MGMT_STR;
    case CDX_C_ATT_NOT_DEFINED :
        return "";             
    default :                  
        return CDX_C_WRONGVALUE_STR;
    }                          
}                              


/*
* @FUNCTION XMLEscape
*
* @DESCRIPTION This function unbinds to the server.
*
* @ABSTRACT
* This function escapes the < > & ' " characters that are
* reserved in XML. The output string is allocated by this
* function and must be freed by the caller.
*
* @RETURNVALUE CSILONG
* The return code (CSISUCCESS/CSIFAILED)
*
* @PARAMETER
* IN  CSICHAR *szIn             The string to escape
* OUT CSICHAR **pszOut          The escaped string
* OUT CSILONG *lReturnCode      The return code
* OUT CSILONG *lReasonCode      The reason code
*
* @SEEALSO
*/

static CSILONG XMLEscape(IN  CSICHAR *szIn
                        ,OUT CSICHAR **pszOut
                        ,OUT CSILONG *plReturnCode
                        ,OUT CSILONG *plReasonCode
                        )
{
    size_t i;
    CSILONG lLength;
    CSICHAR *szPtr;

    /* no string, no conversion */
    if (NULL == szIn)
    {
        *plReturnCode = CSISUCCESS;
        *plReasonCode = OS_ERR_NONE;
        *pszOut = NULL;
        return *plReturnCode;
    }

    /* First the length of the new string is calculated */
    for (i = 0, szPtr = szIn, lLength = 1
        ;i < strlen(szIn)
        ;i++, szPtr++
        )
    {
        switch(szPtr[0])
        {
            case '<' :
            case '>' :
                lLength += 4;
                break;       
            case '&' :
                lLength += 5;
                break;       
            case '\'' :
            case '\"' :
                lLength += 6;
                break;       
            default : 
                lLength++;
                break;    
        }
    }
     
    if (CSISUCCESS != OS_mem_alloc(lLength
                                  ,(CSIVOID **) pszOut
                                  ,plReturnCode        
                                  ,plReasonCode
                                  )           
       )
    {   
        *pszOut = NULL;
        return *plReturnCode;
    }
     
    memset(*pszOut, '\0', lLength);

    /* The escaped string is calculated */
    for (i = 0, szPtr = *pszOut
        ;i < strlen(szIn)
        ;i++
        )   
    {
        switch(szIn[i])
        {
            case '<' :
                strcat(szPtr, "&lt;");
                szPtr +=4;
                break;    
            case '>' :
                strcat(szPtr, "&gt;");
                szPtr +=4;
                break;    
            case '&' :
                strcat(szPtr, "&amp;");
                szPtr +=5;
                break;    
            case '\'' :
                strcat(szPtr, "&apos;");
                szPtr +=6;
                break;    
            case '\"' :
                strcat(szPtr, "&quot;");
                szPtr +=6;
                break;    
            default : 
                szPtr[0] = szIn[i];
                szPtr++;
                break;  
        }
    }
     
    return CSISUCCESS;
}


static CSIVOID PrintTabulation(CSILONG lLevel)
{
    CSILONG i;

    for (i = 0; i < lLevel; i++)
        PrintString("\t");
}


static CSIVOID PrintBeginTag(CSILONG lLevel
                     ,CSICHAR *szTagName
                     ,CSICHAR *szFormat
                     ,...
                     )
{
    va_list sArgs;

    PrintTabulation(lLevel); 
    PrintString("<ccncsi:");
    PrintString(szTagName);
    PrintString("\n");

    va_start(sArgs, szFormat);
    while (*szFormat)
    {
        CSICHAR *szName;
        CSICHAR *szString;
        CSICHAR *szEscString;
        CSILONG lReturnCode, lReasonCode;

        szName = va_arg(sArgs, CSICHAR *);
        PrintTabulation(lLevel + 1);
        PrintString(szName);
        PrintString("=\"");
        switch(*szFormat++)
        {
            case 's' : /* string */
                szString = va_arg(sArgs, CSICHAR *);
                if (CSISUCCESS == XMLEscape(szString
                                           ,&szEscString
                                           ,&lReturnCode
                                           ,&lReasonCode
                                           )
                   )
                {
                    if (strlen(szEscString) > 0)
                       PrintString(szEscString);
                    OS_mem_free((CSIVOID **) &szEscString
                               ,&lReturnCode
                               ,&lReasonCode
                               );
                }
                break;
            case 'l' : /* long */
            {
                CSILONG lLong;
                CSICHAR16 szLong;

                lLong = va_arg(sArgs, CSILONG);
                sprintf(szLong, "%ld", lLong);
                PrintString(szLong);
                break;
            }
            case 'b' : /* boolean */
            {
                CSIBOOLEAN bool;

                bool = va_arg(sArgs, CSIBOOLEAN);
                PrintString((CSITRUE == bool) ? "True" : "False");
                break;
            }
            case 'c' : /* CDXRESULT structure */
            {
                CDXRESULT sCDXResult = va_arg(sArgs, CDXRESULT);;
                CSIBOOLEAN FirstTime;

                for (FirstTime = CSITRUE, szString = CDX_list_first(NULL, &sCDXResult)
                    ;NULL != szString
                    ;szString = CDX_list_next(NULL, &sCDXResult)
                    )
                {
                    if (CSISUCCESS == XMLEscape(szString
                                               ,&szEscString
                                               ,&lReturnCode
                                               ,&lReasonCode
                                               )
                       )        
                    {           
                        if (CSITRUE == FirstTime)
                            FirstTime = CSIFALSE;
                        else
                            PrintString("|");
                        if (strlen(szEscString) > 0)
                            PrintString(szEscString);
                        OS_mem_free((CSIVOID **) &szEscString
                                   ,&lReturnCode
                                   ,&lReasonCode
                                   );
                    }
                }
                break;
            }
            case 'r' : /* AdminRights */
            {
                CSILONG lLong, *plRights, i;
                CSIBOOLEAN FirstTime;

                lLong = va_arg(sArgs, CSILONG);
                plRights = va_arg(sArgs, CSILONG *);
                for (FirstTime = CSITRUE,i = 0; i < lLong; i++)
                {
                    if (CSISUCCESS == XMLEscape(PrintAdminRight(plRights[i])
                                               ,&szEscString
                                               ,&lReturnCode
                                               ,&lReasonCode
                                               )
                       )
                    {
                        if (CSITRUE == FirstTime)
                            FirstTime = CSIFALSE;
                        else
                            PrintString("|");
                        if (strlen(szEscString) > 0)
                            PrintString(szEscString);
                        OS_mem_free((CSIVOID **) &szEscString
                                   ,&lReturnCode             
                                   ,&lReasonCode
                                   );
                    }
                }
                break;
            }
        }
        PrintString("\"\n");
    }
    va_end(sArgs);

    PrintTabulation(lLevel);
    PrintString(">\n");
}


static CSIVOID PrintEndTag(CSILONG lLevel
                   ,CSICHAR *szTagName
                   )
{
    PrintTabulation(lLevel);
    PrintString("</ccncsi:");
    PrintString(szTagName);
    PrintString(">\n");
}


CSIVOID PrintErrorMessage(CSILONG lId
                         ,CSICHAR *szMessage
                         )                  
{
    CSICHAR *szEscString;
    CSILONG lReturnCode, lReasonCode;

    PrintBeginTag(1, "ERROR", "l", "id", lId);
    PrintTabulation(2);
    PrintString("Description=\"");
    if (CSISUCCESS == XMLEscape(szMessage
                               ,&szEscString
                               ,&lReturnCode
                               ,&lReasonCode
                               )
       )
    {
        if (strlen(szEscString) > 0)
            PrintString(szEscString);
        OS_mem_free((CSIVOID **) &szEscString
                   ,&lReturnCode
                   ,&lReasonCode
                   );
    }
    PrintString("\"\n");
    PrintEndTag(1, "ERROR");
}


CSIVOID PrintAdminServiceBegin(CSILONG lReturnCode)
{
    if (NULL != szAction)
    {
        PrintBeginTag(0
                     ,"ADMIN_SERVICE"
                     ,"sss" /* 3 strings sent */
                   /* name            value */
                     ,"xmlns:ccncsi", "http://www.ccncsi.net/schema"
                     ,"name"        ,  szAction
                     ,"return_code" ,  PrintReturnCode(lReturnCode)
                     );
    }
    else
    {
        PrintBeginTag(0
                     ,"ADMIN_SERVICE"
                     ,"sss" /* 3 strings sent */
                   /* name            value */
                     ,"xmlns:ccncsi", "http://www.ccncsi.net/schema"
                     ,"name"        ,  " "
                     ,"return_code" ,  PrintReturnCode(lReturnCode)
                     );
    }
}


CSIVOID PrintAdminServiceEnd(CSIVOID)
{
    PrintEndTag(0, "ADMIN_SERVICE");
}


CSIVOID PrintDomainBegin(CSICHAR *szWebDomain)
{
    CSICHAR *szDomain = strchr(getenv(CDX_C_ENV_ROOT), '=') + 1;

    PrintBeginTag(1
                 ,"DOMAIN"
                 ,"ss"
                 ,"domain"   , szDomain
                 ,"webdomain", szWebDomain
                 );
}


CSIVOID PrintDomainEnd(CSIVOID)
{
    PrintEndTag(1, "DOMAIN");
}


CSIVOID PrintSiteBegin(CSILONG lLevel
                      ,CDXSITEATTINFO sAtt
                      )                      
{
    extern CSICHAR *CDX_C_ATT_DESCRIPTION[];
    CSICHAR48 szDescriptionString;
                              
    strcpy(szDescriptionString, CDX_C_ATT_DESCRIPTION[0]);
    to_lower(szDescriptionString);
    PrintBeginTag(lLevel
                 ,"SITE"
                 ,"ssss"
                 ,"site"             , sAtt.szName
                 ,"ccnMailServerSite"         , sAtt.szLcmsName
                 ,"website"          , sAtt.szWebSiteName
                 ,szDescriptionString, sAtt.szDescription
                 );
}


CSIVOID PrintSiteEnd(CSILONG lLevel)
{
    PrintEndTag(lLevel, "SITE");
}


CSIVOID PrintGatewayBegin(CSILONG lLevel
                         ,CDXGTWATTINFO sAtt
                         )                      
{   
    extern CSICHAR *CDX_C_ATT_ADDRESS_TYPE[];
    extern CSICHAR *CDX_C_ATT_ADDRESS[];
    extern CSICHAR *CDX_C_ATT_DESCRIPTION[];
    CSICHAR48 szDescriptionString;

    strcpy(szDescriptionString, CDX_C_ATT_DESCRIPTION[0]);
    to_lower(szDescriptionString);
    PrintBeginTag(lLevel
                 ,"GTW"
                 ,"sscsss"
                 ,"gtw",                      sAtt.szName
                 ,"hostname",                 sAtt.szHostName
                 ,"modelist",                 sAtt.sGtwModeList
                 ,CDX_C_ATT_ADDRESS_TYPE[0],  sAtt.szAddressType
                 ,CDX_C_ATT_ADDRESS[0],       sAtt.szAddress
                 ,szDescriptionString,        sAtt.szDescription
                 );
}

CSIVOID PrintGatewayBeginM(CSILONG lLevel
                         ,CDXGTWATTINFO sAtt
                         ,CSICHAR *modelist
			 )                  
{  
    extern CSICHAR *CDX_C_ATT_ADDRESS_TYPE[];
    extern CSICHAR *CDX_C_ATT_ADDRESS[];
    extern CSICHAR *CDX_C_ATT_DESCRIPTION[];
    CSICHAR48 szDescriptionString;

    strcpy(szDescriptionString, CDX_C_ATT_DESCRIPTION[0]);
    to_lower(szDescriptionString);
    PrintBeginTag(lLevel
                 ,"GTW" 
                 ,"ssssss"
                 ,"gtw",                      sAtt.szName
                 ,"hostname",                 sAtt.szHostName
                 ,"modelist",                 modelist 
                 ,CDX_C_ATT_ADDRESS_TYPE[0],  sAtt.szAddressType
                 ,CDX_C_ATT_ADDRESS[0],       sAtt.szAddress
                 ,szDescriptionString,        sAtt.szDescription
                 );
}

CSIVOID PrintLcmsBegin(CSILONG lLevel
                         ,CDXLCMSATTINFO sAtt
                         )                  
{  
    extern CSICHAR *CDX_C_ATT_ADDRESS_TYPE[];
    extern CSICHAR *CDX_C_ATT_ADDRESS[];
    extern CSICHAR *CDX_C_ATT_DESCRIPTION[];
    CSICHAR48 szDescriptionString;

    strcpy(szDescriptionString, CDX_C_ATT_DESCRIPTION[0]);
    to_lower(szDescriptionString);
    PrintBeginTag(lLevel
                 ,"LCMS" 
                 ,"sscsss"
                 ,"lcms",                     sAtt.szName
                 ,"hostname",                 sAtt.szHostName
                 ,"modelist",                 sAtt.sGtwModeList
                 ,CDX_C_ATT_ADDRESS_TYPE[0],  sAtt.szAddressType
                 ,CDX_C_ATT_ADDRESS[0],       sAtt.szAddress
                 ,szDescriptionString,        sAtt.szDescription
                 );
}

CSIVOID PrintLcmsBeginM(CSILONG lLevel
                         ,CDXLCMSATTINFO sAtt
                         ,CSICHAR *modelist
			 )
{       
    extern CSICHAR *CDX_C_ATT_ADDRESS_TYPE[];
    extern CSICHAR *CDX_C_ATT_ADDRESS[];
    extern CSICHAR *CDX_C_ATT_DESCRIPTION[];
    CSICHAR48 szDescriptionString;
        
    strcpy(szDescriptionString, CDX_C_ATT_DESCRIPTION[0]);
    to_lower(szDescriptionString);
    PrintBeginTag(lLevel
                 ,"LCMS"
                 ,"ssssss"
                 ,"lcms",                     sAtt.szName
                 ,"hostname",                 sAtt.szHostName
                 ,"modelist",                 modelist
                 ,CDX_C_ATT_ADDRESS_TYPE[0],  sAtt.szAddressType
                 ,CDX_C_ATT_ADDRESS[0],       sAtt.szAddress
                 ,szDescriptionString,        sAtt.szDescription
                 );
}       


CSIVOID PrintLcmsEnd(CSILONG lLevel)
{
    PrintEndTag(lLevel, "LCMS");
}

CSIVOID PrintGatewayEnd(CSILONG lLevel)
{
    PrintEndTag(lLevel, "GTW");
}


CSIVOID PrintNS50Begin(CSILONG lLevel
                         ,CDXFWLATTINFO sAtt
                         )                  
{  
    extern CSICHAR *CDX_C_ATT_ADDRESS_TYPE[];
    extern CSICHAR *CDX_C_ATT_ADDRESS[];
    extern CSICHAR *CDX_C_ATT_DESCRIPTION[];
    CSICHAR48 szDescriptionString;

    strcpy(szDescriptionString, CDX_C_ATT_DESCRIPTION[0]);
    to_lower(szDescriptionString);
    PrintBeginTag(lLevel
                 ,"NS50" 
                 ,"sscsss"
                 ,"ns50",                     sAtt.szName
                 ,"hostname",                 sAtt.szHostName
                 ,"modelist",                 sAtt.sGtwModeList
                 ,CDX_C_ATT_ADDRESS_TYPE[0],  sAtt.szAddressType
                 ,CDX_C_ATT_ADDRESS[0],       sAtt.szAddress
                 ,szDescriptionString,        sAtt.szDescription
                 );
}


CSIVOID PrintNS50End(CSILONG lLevel)
{
    PrintEndTag(lLevel, "NS50");
}


CSIVOID PrintRapMgmtRap(CSICHAR *szRapName
                       ,CSICHAR *szAppd
                       ,CSICHAR *szRunMode
		       ,CSIBOOLEAN CanBeManaged
                       ,CSICHAR *szRapState 
                       )
{
    PrintBeginTag(4
                 ,"RAP"
                 ,"sssss"
                 ,"rap"        , szRapName
                 ,"application", szAppd
                 ,"runmode"    , szRunMode
		 ,"mgmt"       , CanBeManaged ? "yes" : "no"
                 ,"running"    , szRapState
                 );
    PrintEndTag(4, "RAP");

}


CSIVOID PrintMonitorHostBegin(CSICHAR *szName
                             ,CSICHAR *szAlert
                             ,CSICHAR *szDate
                             ,CSICHAR *szTime
                             )
{
    CSICHAR48 szDateTime;

    sprintf(szDateTime, "%s %s", szDate, szTime);
    PrintBeginTag(1
                 ,"HOST"
                 ,"sss"
                 ,"name",  szName
                 ,"alert", szAlert
                 ,"date",  szDateTime
                 );
}


CSIVOID PrintMonitorHostEnd(CSIVOID)
{
    PrintEndTag(1, "HOST");
}


CSIVOID PrintMonitorAlert(CSICHAR *szName
                         ,CSICHAR *szAlert
                         ,CSICHAR *szObject
                         ,CSICHAR *szMessage
                         )
{
    PrintBeginTag(3
                 ,"ALERT"
                 ,"ss"
                 ,"name",  szName
                 ,"alert", szAlert
                 );

    if (   NULL != szObject
        && szObject[0] != '\0'
       )
    {
        CSICHAR *szEscString;
        CSILONG lReturnCode, lReasonCode;

        if (CSISUCCESS == XMLEscape(szObject
                                   ,&szEscString
                                   ,&lReturnCode
                                   ,&lReasonCode
                                   )            
           )              
        {   
            if (strlen(szEscString) > 0)
                PrintString(szEscString);
            PrintString(" ");
            OS_mem_free((CSIVOID **) &szEscString
                       ,&lReturnCode             
                       ,&lReasonCode
                       );           
        }               
    }

    if (   NULL != szMessage
        && szMessage[0] != '\0'
       )
    {
        CSICHAR *szEscString;
        CSILONG lReturnCode, lReasonCode;

        if (CSISUCCESS == XMLEscape(szMessage
                                   ,&szEscString
                                   ,&lReturnCode
                                   ,&lReasonCode
                                   )            
           )
        {   
            if (strlen(szEscString) > 0)
               PrintString(szEscString);
            OS_mem_free((CSIVOID **) &szEscString
                       ,&lReturnCode
                       ,&lReasonCode
                       );           
        }
    }

    PrintEndTag(3, "ALERT");
}


CSIVOID PrintPlatform(CSILONG lLevel, CDXPLFATTINFO sAtt)
{
    extern CSICHAR *CDX_C_ATT_PLATFORM_NAME[];
    extern CSICHAR *CDX_C_ATT_ADDRESS_TYPE[];
    extern CSICHAR *CDX_C_ATT_ADDRESS[];
    extern CSICHAR *CDX_C_ATT_OPERATIONAL_STATUS[];
    extern CSICHAR *CDX_C_ATT_AUTHORIZED_SECURITY_MECHANISM[];
    extern CSICHAR *CDX_C_ATT_DEFAULT_SECURITY_MECHANISM[];
    extern CSICHAR *CDX_C_ATT_DATA_REPRESENTATION_RULES[];
    extern CSICHAR *CDX_C_ATT_DEFAULT_CODE_PAGE[];

    PrintBeginTag(lLevel
                 ,"PLF"
                 ,"ssssssss"
                 ,CDX_C_ATT_PLATFORM_NAME[0]
                 ,sAtt.szName
                 ,CDX_C_ATT_ADDRESS_TYPE[0]
                 ,sAtt.szAddressType
                 ,CDX_C_ATT_ADDRESS[0]
                 ,sAtt.szAddress
                 ,CDX_C_ATT_OPERATIONAL_STATUS[0]
                 ,PrintOperationalStatus(sAtt.lOperationalStatus)
                 ,CDX_C_ATT_AUTHORIZED_SECURITY_MECHANISM[0]
                 ,sAtt.szAuthSecMech
                 ,CDX_C_ATT_DEFAULT_SECURITY_MECHANISM[0]
                 ,sAtt.szDefaultSecMech
                 ,CDX_C_ATT_DATA_REPRESENTATION_RULES[0]
                 ,sAtt.szDRR
                 ,CDX_C_ATT_DEFAULT_CODE_PAGE[0]
                 ,PrintCodePage(sAtt.lDefaultCodePage)
                 );

    PrintEndTag(lLevel, "PLF");
}


CSIVOID PrintMonitorServiceBegin(CSICHAR *szName
                                ,CSICHAR *szAlert
                                ,CSICHAR *szDate
                                ,CSICHAR *szTime
                                )
{
    CSICHAR48 szDateTime;

    sprintf(szDateTime, "%s %s", szDate, szTime);
    PrintBeginTag(2
                 ,"SERVICE"
                 ,"sss" 
                 ,"name",  szName
                 ,"alert", szAlert
                 ,"date",  szDateTime
                 );
}


CSIVOID PrintMonitorServiceEnd(CSIVOID)
{
    PrintEndTag(2, "SERVICE");
}


CSIVOID PrintWebService(CSILONG lLevel, CDXWEBSVCATTINFO sAtt)
{
    extern CSICHAR *CDX_C_ATT_WEB_SERVICE[];
    extern CSICHAR *CDX_C_ATT_TICKET_FREE_ACCESS[];
    extern CSICHAR *CDX_C_ATT_WEB_SERVICE_APP[];
    extern CSICHAR *CDX_C_ATT_WEB_SERVICE_ACL[];
    extern CSICHAR *CDX_C_ATT_WEB_HOST[];
    CSICHAR48 szWebSvcString;
    CSICHAR48 szWebHostString;

    strcpy(szWebSvcString, CDX_C_ATT_WEB_SERVICE[0]);
    to_lower(szWebSvcString);
    strcpy(szWebHostString, CDX_C_ATT_WEB_HOST[0]);
    to_lower(szWebHostString);
    PrintBeginTag(lLevel
                 ,"WEBSRV"
                 ,"slscs"
                 ,szWebSvcString,                  sAtt.szName
                 ,CDX_C_ATT_TICKET_FREE_ACCESS[0], sAtt.lFreeAccess
                 ,CDX_C_ATT_WEB_SERVICE_APP[0],    sAtt.szApplicationName
                 ,CDX_C_ATT_WEB_SERVICE_ACL[0],    sAtt.sAclList
                 ,szWebHostString,                 sAtt.szWebHostName
                 );
    PrintEndTag(lLevel, "WEBSRV");
}


CSIVOID PrintWebHostBegin(CSILONG lLevel, CDXWEBHOSTATTINFO sAtt)
{
    extern CSICHAR *CDX_C_ATT_WEB_HOST[];
    extern CSICHAR *CDX_C_ATT_SITE_NAME[];
    extern CSICHAR *CDX_C_ATT_HOST_NAME[];
    extern CSICHAR *CDX_C_ATT_PROXY_REMOTE[];
    extern CSICHAR *CDX_C_ATT_IS_HTTPS[];
    extern CSICHAR *CDX_C_ATT_DESCRIPTION[];
    CSICHAR48 szWebHostString;
    CSICHAR48 szDescriptionString;

    strcpy(szWebHostString, CDX_C_ATT_WEB_HOST[0]);
    to_lower(szWebHostString);
    strcpy(szDescriptionString, CDX_C_ATT_DESCRIPTION[0]);
    to_lower(szDescriptionString);
    PrintBeginTag(lLevel
                 ,"WEBHOST"
                 ,"ssssls"
                 ,szWebHostString,           sAtt.szName
                 ,CDX_C_ATT_SITE_NAME[0],    sAtt.szSiteName
                 ,CDX_C_ATT_HOST_NAME[0],    sAtt.szHostName
                 ,CDX_C_ATT_PROXY_REMOTE[0], sAtt.szProxyRemoteName
                 ,CDX_C_ATT_IS_HTTPS[0],     sAtt.isHttps
                 ,szDescriptionString,       sAtt.szDescription
                 );
}


CSIVOID PrintWebHostEnd(CSILONG lLevel)
{
    PrintEndTag(lLevel, "WEBHOST");
}


CSIVOID PrintWebApplication(CSILONG lLevel
                           ,CSICHAR *szWebHostName
                           ,CSICHAR *szApplicationName
                           )
{
    extern CSICHAR *CDX_C_ATT_WEB_HOST[];
    extern CSICHAR *CDX_C_ATT_APPLICATION_NAME[];
    CSICHAR48 szString;

    strcpy(szString, CDX_C_ATT_WEB_HOST[0]);
    to_lower(szString);
    PrintBeginTag(lLevel
                 ,"APP"
                 ,"ss"
                 ,szString,                      szWebHostName
                 ,CDX_C_ATT_APPLICATION_NAME[0], szApplicationName
                 );
    PrintEndTag(lLevel, "APP");
}


CSIVOID PrintApplicationTagBegin(CSILONG lLevel
                                ,CSICHAR *szApplicationName
                                )
{
    extern CSICHAR *CDX_C_ATT_APPLICATION_NAME[];

    PrintBeginTag(lLevel
                 ,"APP" 
                 ,"s" 
                 ,CDX_C_ATT_APPLICATION_NAME[0], szApplicationName
                 );
}


CSIVOID PrintApplicationTagEnd(CSILONG lLevel)
{
    PrintEndTag(lLevel, "APP");
}

CSIVOID PrintQoS(CSIQOS sAtt)
{
    PrintBeginTag(5
                 ,"QOS"
                 ,"slssssssssssss"
                 ,QOS_CLASS_OF_TRAFFIC_TYPE, sAtt.CoT
                 ,QOS_PRIORITY_TYPE,         sAtt.Priority
                 ,QOS_INTEGRITY_TYPE,        sAtt.Integrity ? "required" 
                                                            : "not_required"
                 ,QOS_CONFIDENTIALITY_TYPE,  sAtt.Confidentiality ? "required"
                                                                  : "not_required"
                 ,QOS_COMPRESSION_TYPE,      sAtt.Compression ? "required"
                                                              : "not_required"
                 ,QOS_COMPRESSION_ID_TYPE,   sAtt.CompressionId
                 ,QOS_DEGRADED_MODE_TYPE,    sAtt.DegradedMode ? "allowed" 
                                                               : "not_allowed"
                 ,QOS_REPLYTO_Q_TYPE,        sAtt.ReplyToQ

                 ,"ReportException"
                 ,(   (sAtt.ReportRequest & CSIMQRO_EXCEPTION_WITH_FULL_DATA)
                   || (sAtt.ReportRequest & CSIMQRO_EXCEPTION_WITH_DATA)
                  )
                      ? "yes_with_data"
                      : ((sAtt.ReportRequest & CSIMQRO_EXCEPTION) ? "yes" : "no")

                 ,"ReportExpiration"
                 ,(   (sAtt.ReportRequest & CSIMQRO_EXPIRATION_WITH_FULL_DATA)
                   || (sAtt.ReportRequest & CSIMQRO_EXPIRATION_WITH_DATA)    
                  )
                      ? "yes_with_data"
                      : ((sAtt.ReportRequest & CSIMQRO_EXPIRATION) ? "yes" : "no")
            
                 ,"ReportConfirmArrival"
                 ,(   (sAtt.ReportRequest & CSIMQRO_COA_WITH_FULL_DATA)
                   || (sAtt.ReportRequest & CSIMQRO_COA_WITH_DATA)     
                  )
                      ? "yes_with_data"
                      : ((sAtt.ReportRequest & CSIMQRO_COA) ? "yes" : "no")

                 ,"ReportConfirmDelivery"
                 ,(   (sAtt.ReportRequest & CSIMQRO_COD_WITH_FULL_DATA)
                   || (sAtt.ReportRequest & CSIMQRO_COD_WITH_DATA)     
                  )                                                           
                      ? "yes_with_data"                                       
                      : ((sAtt.ReportRequest & CSIMQRO_COD) ? "yes" : "no")

                 ,"MessageId"
                 ,CSIMQRO_PASS_MSG_ID & sAtt.ReportRequest ? "pass" : "new" 

                 ,"CorrelationId"
                 ,CSIMQRO_COPY_MSG_ID_TO_CORREL_ID & sAtt.ReportRequest
                      ? "copy" : "pass"
                 );
    PrintEndTag(5, "QOS");
}


CSIVOID PrintRap(CDXRAPATTINFO sAtt)
{
    extern CSICHAR *CDX_C_ATT_RAP_NAME[];
    extern CSICHAR *CDX_C_ATT_ADDRESS[];
    extern CSICHAR *CDX_C_ATT_ADDRESS_TYPE[];

    PrintBeginTag(5
                 ,"RAP"
                 ,"sss"
                 ,CDX_C_ATT_RAP_NAME[0],     sAtt.szName
                 ,CDX_C_ATT_ADDRESS[0],      sAtt.szAddress
                 ,CDX_C_ATT_ADDRESS_TYPE[0], sAtt.szAddressType
                 );
    PrintEndTag(5, "RAP");
}


CSIVOID PrintQueue(CDXQUEATTINFO sAtt)
{
    extern CSICHAR *CDX_C_ATT_QUEUE_NAME[];
    extern CSICHAR *CDX_C_ATT_APPLICATION_NAME[];
    extern CSICHAR *CDX_C_ATT_MAX_QUEUE_DEPTH[];
    extern CSICHAR *CDX_C_ATT_QUEUE_ACL_READ[];
    extern CSICHAR *CDX_C_ATT_QUEUE_ACL_BROWSE[];
    extern CSICHAR *CDX_C_ATT_QUEUE_ACL_WRITE[];
    extern CSICHAR *CDX_C_ATT_TRIGGER_ENABLED[];
    extern CSICHAR *CDX_C_ATT_TRIGGER_TYPE[];

    PrintBeginTag(5
                 ,"QUE"
                 ,"sslcccbs"
                 ,CDX_C_ATT_QUEUE_NAME[0],       sAtt.szName
                 ,CDX_C_ATT_APPLICATION_NAME[0], sAtt.szAppName
                 ,CDX_C_ATT_MAX_QUEUE_DEPTH[0],  sAtt.lMaxDepth
                 ,CDX_C_ATT_QUEUE_ACL_READ[0],   sAtt.sACLRead
                 ,CDX_C_ATT_QUEUE_ACL_BROWSE[0], sAtt.sACLBrowse
                 ,CDX_C_ATT_QUEUE_ACL_WRITE[0],  sAtt.sACLWrite
                 ,CDX_C_ATT_TRIGGER_ENABLED[0],  sAtt.TriggerEnabled
                 ,CDX_C_ATT_TRIGGER_TYPE[0],     sAtt.szTriggerType
                 );
 
    PrintEndTag(5, "QUE");
}


CSIVOID PrintService(CDXSVCATTINFO sAtt)
{
    extern CSICHAR *CDX_C_ATT_SERVICE_NAME[];
    extern CSICHAR *CDX_C_ATT_APPLICATION_NAME[];
    extern CSICHAR *CDX_C_ATT_SERVICE_ACL[];
    extern CSICHAR *CDX_C_ATT_SERVICE_TIMEOUT[];
    extern CSICHAR *CDX_C_ATT_STATICALLY_ADVERTIZED[];

    PrintBeginTag(5
                 ,"SRV"
                 ,"ssclb"
                 ,CDX_C_ATT_SERVICE_NAME[0],          sAtt.szName
                 ,CDX_C_ATT_APPLICATION_NAME[0],      sAtt.szAppName
                 ,CDX_C_ATT_SERVICE_ACL[0],           sAtt.sACL
                 ,CDX_C_ATT_SERVICE_TIMEOUT[0],       sAtt.lTimeout
                 ,CDX_C_ATT_STATICALLY_ADVERTIZED[0], sAtt.StaticallyAdvertized
                 );
    PrintEndTag(5, "SRV");
}


CSIVOID PrintProfile(CSILONG lLevel
                    ,CSICHAR *szName
                    ,CDXRESULT sAdminToolList
                    ,CDXRESULT sAppdList
                    ,CSILONG lAdminRole
                    ,CSILONG lAdminRight
                    ,CDXRESULT sUsageModeList
                    ,CSICHAR *szAppDomain
                    )
{
    extern CSICHAR *CDX_C_ATT_PROFILE_ID[];
/*
    extern CSICHAR *CDX_C_ATT_PRF_ADM_TOOL[];
    extern CSICHAR *CDX_C_ATT_PRF_ADM_APPD[];
    extern CSICHAR *CDX_C_ATT_PRF_ADM_ROLE[];
    extern CSICHAR *CDX_C_ATT_PRF_ADM_RIGHTS[];
    extern CSICHAR *CDX_C_ATT_PRF_USAGE_MODE[];
*/

    PrintBeginTag(lLevel
                 ,"PRF"
                 ,"sccsscs"
                 ,CDX_C_ATT_PROFILE_ID[0], szName
                 ,"toollist",              sAdminToolList
                 ,"appdomlist",            sAppdList
                 ,"role",                  PrintAdminRole(lAdminRole)
                 ,"right",                 PrintAdminRight(lAdminRight)
                 ,"modelist",              sUsageModeList
                 ,"appdom",                szAppDomain
                 );
    PrintEndTag(lLevel, "PRF");
}


CSIVOID PrintApplicationBegin(CSILONG lLevel
                             ,CSIBOOLEAN printAttributes
                             ,CSIVOID *pVoid
                             )
{
    extern CSICHAR *CDX_C_ATT_APPLICATION_NAME[];
    extern CSICHAR *CDX_C_ATT_PLATFORM_NAME[];
    extern CSICHAR *CDX_C_ATT_RAP_NAME[];
    extern CSICHAR *CDX_C_ATT_ADDRESS_TYPE[];
    extern CSICHAR *CDX_C_ATT_ADDRESS[];
    extern CSICHAR *CDX_C_ATT_APPLICATION_TYPE[];
    extern CSICHAR *CDX_C_ATT_APPLICATION_MODE[];
    extern CSICHAR *CDX_C_ATT_OPERATIONAL_STATUS[];
    extern CSICHAR *CDX_C_ATT_APPLICATION_ACTIVATION_MODE[];
    extern CSICHAR *CDX_C_ATT_APPLICATION_EXCHANGE_MODE[];
    extern CSICHAR *CDX_C_ATT_AUTHORIZED_SECURITY_MECHANISM[];
    extern CSICHAR *CDX_C_ATT_DEFAULT_SECURITY_MECHANISM[];
    extern CSICHAR *CDX_C_ATT_RUN_MODE[];

    if (!printAttributes)
        PrintBeginTag(lLevel
                     ,"APP"
                     ,"s"
                     ,CDX_C_ATT_APPLICATION_NAME[0], pVoid
                     );
    else
    {
        CDXAPPATTINFO sAtt;

        memcpy(&sAtt, pVoid, sizeof(sAtt));
        
        PrintBeginTag(lLevel
                     ,"APP"
                     ,"sssssssssssss"
                     ,CDX_C_ATT_APPLICATION_NAME[0], sAtt.szName
                     ,CDX_C_ATT_PLATFORM_NAME[0],    sAtt.szPlfName
                     ,CDX_C_ATT_RAP_NAME[0],         sAtt.szRapName
                     ,CDX_C_ATT_ADDRESS_TYPE[0],     sAtt.szAddressType
                     ,CDX_C_ATT_ADDRESS[0],          sAtt.szAddress
                     ,CDX_C_ATT_APPLICATION_TYPE[0], sAtt.szType
                     ,CDX_C_ATT_APPLICATION_MODE[0], sAtt.szMode

                     ,CDX_C_ATT_OPERATIONAL_STATUS[0]
                     ,PrintOperationalStatus(sAtt.lOperationalStatus)

                     ,CDX_C_ATT_APPLICATION_ACTIVATION_MODE[0]
                     ,sAtt.szActivationMode

                     ,CDX_C_ATT_APPLICATION_EXCHANGE_MODE[0]
                     ,sAtt.szExchangeMode

                     ,CDX_C_ATT_AUTHORIZED_SECURITY_MECHANISM[0]
                     ,sAtt.szAuthSecMech

                     ,CDX_C_ATT_DEFAULT_SECURITY_MECHANISM[0]
                     ,sAtt.szDefaultSecMech

		     ,CDX_C_ATT_RUN_MODE[0]
                     ,PrintMode(sAtt.lRunMode)

                     );
    }
}


CSIVOID PrintApplicationEnd(CSILONG lLevel)
{
    PrintEndTag(lLevel, "APP");
}


CSIVOID PrintApplicationDomainBegin(CSILONG lLevel, CSICHAR *szName)
{
    PrintBeginTag(lLevel
                 ,"APPDOM"
                 ,"s"
                 ,"appdom"
                 ,szName
                 );
}


CSIVOID PrintApplicationDomainEnd(CSILONG lLevel)
{
    PrintEndTag(lLevel, "APPDOM");
}


CSIVOID PrintAdminToolBegin(CSILONG lLevel, CSICHAR *szName)
{
    extern CSICHAR *CDX_C_ATT_PRF_ADM_TOOL[];

    PrintBeginTag(lLevel
                 ,"TOOL"
                 ,"s"
                 ,CDX_C_ATT_PRF_ADM_TOOL[0]
                 ,szName
                 );
}


CSIVOID PrintAdminToolEnd(CSILONG lLevel)
{
    PrintEndTag(lLevel, "TOOL");
}


CSIVOID PrintRight(CSILONG lLevel
                  ,CSILONG lNbRights
                  ,CSILONG *plRights
                  )
{
    PrintBeginTag(lLevel
                 ,"RIGHT"
                 ,"r"
                 ,"rightlist"
                 ,lNbRights
                 ,plRights
                 );
    PrintEndTag(lLevel, "RIGHT");
}


static CSIVOID HandleError(CSILONG lErrorId, ...)
{
    va_list arguments;
    CSICHAR1024 szMsg;
    CSILONG lReturnCode;
    CSILONG lReasonCode;

    va_start(arguments, lErrorId);
    vsprintf(szMsg
            ,COM_ERROR_MSGS[lErrorId - 1]
            ,arguments
            );        
    va_end(arguments);

    PrintAdminServiceBegin(CSIFAILED);
    PrintErrorMessage(lErrorId, szMsg);
    PrintAdminServiceEnd();

    if (CSITRUE == CDXBind)
    {
        CDX_unbind(NULL, &lReturnCode, &lReasonCode);
        CDXBind = CSIFALSE;
    } 
      
    if (CSITRUE == OSInit)
    { 
        OS_term(&lReturnCode, &lReasonCode);
        OSInit = CSIFALSE;
    } 
      
    exit(1);
}     


CSIVOID PrintSiteList(CSIVOID)
{
    CDXRESULT sSiteList = {CDXRESULT_DEFAULT};
    CSILONG lReturnCode;
    CSILONG lReasonCode;
    CSICHAR *szSiteName;
    CSILONG lNbSite;
    CDXSITEATTINFO *psSiteAtt;
    CSILONG i;

    if (CSISUCCESS != CDX_listSites2(NULL
                                    ,&sSiteList
                                    ,&lReturnCode
                                    ,&lReasonCode
                                    )
       )
    {
        HandleError(COM_ERROR_CDX
                   ,"CDX_listSites2"
                   ,lReturnCode
                   ,lReasonCode
                   );
    }

    for (lNbSite = 0, szSiteName = CDX_list_first(NULL, &sSiteList)
        ;NULL != szSiteName
        ;szSiteName = CDX_list_next(NULL, &sSiteList)
        )
    {
        lNbSite++;
    }

    if (CSISUCCESS != OS_mem_alloc(sizeof(*psSiteAtt) * lNbSite          
                                  ,(CSIVOID **) &psSiteAtt
                                  ,&lReturnCode
                                  ,&lReasonCode
                                  )            
       )
    {   
        CDX_free(NULL, &sSiteList);
        HandleError(COM_ERROR_OS
                   ,"OS_mem_alloc"
                   ,lReturnCode   
                   ,lReasonCode
                   );          
    }
     
    for (lNbSite = 0, szSiteName = CDX_list_first(NULL, &sSiteList)
        ;NULL != szSiteName
        ;lNbSite++, szSiteName = CDX_list_next(NULL, &sSiteList)
        )
    {
        if (CSISUCCESS != CDX_getAttributesOfSite(NULL
                                                 ,szSiteName
                                                 ,&psSiteAtt[lNbSite]
                                                 ,&lReturnCode  
                                                 ,&lReasonCode
                                                 )
           )
        {
            CSILONG lLocalReturnCode;
            CSILONG lLocalReasonCode;

            CDX_free(NULL, &sSiteList);
            OS_mem_free((CSIVOID **) &psSiteAtt
                       ,&lLocalReturnCode
                       ,&lLocalReasonCode
                       );                
            HandleError(COM_ERROR_CDX
                       ,"CDX_getAttributesOfSite"
                       ,lReturnCode           
                       ,lReasonCode
                       );          
        }
    }
     
    CDX_free(NULL, &sSiteList);

    PrintAdminServiceBegin(CSISUCCESS);
    PrintDomainBegin(szWebDomain);
    for (i = 0; i < lNbSite; i++)
    {
        PrintSiteBegin(2, psSiteAtt[i]);
        PrintSiteEnd(2);
    }
    PrintDomainEnd();
    PrintAdminServiceEnd();

    OS_mem_free((CSIVOID **) &psSiteAtt, &lReturnCode, &lReasonCode);           
    return;
}

static CSIVOID free_CDXGTWITEM(CDXGTWITEM **ptsGwList, CSILONG lNbrGWs)
   {
   CSILONG lReturn, lReason, i;

   for (i = 0; i < lNbrGWs; i++)
      CDX_free(NULL, &((*ptsGwList)[i].sGtwModeList));

   OS_mem_free((CSIVOID **)ptsGwList, &lReturn, &lReason);
   }
                      
CSIVOID PrintGatewayList(CSIVOID)
{
    CDXRESULT sSiteList = {CDXRESULT_DEFAULT};   
    CSILONG lReturnCode;
    CSILONG lReasonCode;
    CSICHAR *szSiteName;
    CSILONG lNbSite;
    CDXSITEATTINFO *psSiteAtt;
    CSILONG i, j;
    CDXGTWITEM *psGatewayItem;
    CSILONG lNbGateway;
    CDXGTWATTINFO *psGatewayAtt;

    if (CSISUCCESS != CDX_listSites2(NULL
                                    ,&sSiteList
                                    ,&lReturnCode
                                    ,&lReasonCode
                                    )
       )
    {
        HandleError(COM_ERROR_CDX
                   ,"CDX_lisSites2"
                   ,lReturnCode
                   ,lReasonCode
                   );          
    }

    for (lNbSite = 0, szSiteName = CDX_list_first(NULL, &sSiteList)
        ;NULL != szSiteName
        ;szSiteName = CDX_list_next(NULL, &sSiteList)
        )
    {
        lNbSite++;
    }

    if (CSISUCCESS != OS_mem_alloc(sizeof(*psSiteAtt) * lNbSite
                                  ,(CSIVOID **)&psSiteAtt
                                  ,&lReturnCode
                                  ,&lReasonCode
                                  )
       )
    {
        CDX_free(NULL, &sSiteList);
        HandleError(COM_ERROR_OS    
                   ,"OS_mem_alloc"  
                   ,lReturnCode  
                   ,lReasonCode  
                   );            
    }

    for (szSiteName = CDX_list_first(NULL, &sSiteList), i = 0
        ;NULL != szSiteName
        ;szSiteName = CDX_list_next(NULL, &sSiteList), i++
        )
    {
        if (CSISUCCESS != CDX_getAttributesOfSite(NULL
                                                 ,szSiteName
                                                 ,&(psSiteAtt[i])
                                                 ,&lReturnCode
                                                 ,&lReasonCode
                                                 )
           )          
        {             
            CSILONG lLocalReturnCode;
            CSILONG lLocalReasonCode;
                  
            CDX_free(NULL, &sSiteList);
            OS_mem_free((CSIVOID **)&psSiteAtt
                       ,&lLocalReturnCode
                       ,&lLocalReasonCode
                       );                
            HandleError(COM_ERROR_CDX
                       ,"CDX_getAttributesOfSite"
                       ,lReturnCode
                       ,lReasonCode
                       );
        }
    }

    CDX_free(NULL, &sSiteList);

    if (CSISUCCESS != CDX_listGateways(NULL
                                       ,&psGatewayItem
                                       ,&lNbGateway   
                                       ,&lReturnCode
                                       ,&lReasonCode
                                       )            
       )
    {   
        CSILONG lLocalReturnCode;
        CSILONG lLocalReasonCode;
   
        OS_mem_free((CSIVOID **)&psSiteAtt
                   ,&lLocalReturnCode
                   ,&lLocalReasonCode
                   );
        HandleError(COM_ERROR_CDX
                   ,"CDX_listGateways"
                   ,lReturnCode
                   ,lReasonCode
                   );          
    }
    
    if (CSISUCCESS != OS_mem_alloc(sizeof(*psGatewayAtt) * lNbGateway
                                  ,(CSIVOID **)&psGatewayAtt
                                  ,&lReturnCode
                                  ,&lReasonCode
                                  )            
       )
    {   
        CSILONG lLocalReturnCode;
        CSILONG lLocalReasonCode;

        OS_mem_free((CSIVOID **)&psSiteAtt
                   ,&lLocalReturnCode
                   ,&lLocalReasonCode
                   );
        OS_mem_free((CSIVOID **)&psGatewayItem
                   ,&lLocalReturnCode
                   ,&lLocalReasonCode
                   );
	free_CDXGTWITEM(&psGatewayItem, lNbGateway);

        HandleError(COM_ERROR_OS
                   ,"OS_mem_alloc"
                   ,lReturnCode   
                   ,lReasonCode
                   );          
    }

    for (i = 0; i < lNbGateway; i++)
    {     
        if (CSISUCCESS != CDX_getAttributesOfGateway(NULL
                                                    ,psGatewayItem[i].szSiteName
                                                    ,psGatewayItem[i].szGatewayName
                                                    ,&(psGatewayAtt[i])
                                                    ,&lReturnCode
                                                    ,&lReasonCode
                                                    )
           )
        {
            CSILONG lLocalReturnCode;
            CSILONG lLocalReasonCode;

            OS_mem_free((CSIVOID **)&psSiteAtt
                       ,&lLocalReturnCode
                       ,&lLocalReasonCode
                       );
	    free_CDXGTWITEM(&psGatewayItem, lNbGateway);

            for (j = 0; j < i; j++)
            {
              CDX_free(NULL,&(psGatewayAtt[j].sGtwModeList));
              CDX_free(NULL,&(psGatewayAtt[j].sMonitoringUnavailabilityList));
            }
            OS_mem_free((CSIVOID **)&psGatewayAtt
                       ,&lLocalReturnCode
                       ,&lLocalReasonCode
                       );                
            HandleError(COM_ERROR_CDX
                       ,"CDX_getAttributesOfGateway"
                       ,lReturnCode         
                       ,lReasonCode
                       );          
        }
    }

    free_CDXGTWITEM(&psGatewayItem, lNbGateway);

    PrintAdminServiceBegin(CSISUCCESS);
    PrintDomainBegin(szWebDomain);
/* Begin OR7719 */ 
    for (j = 0; j < lNbSite; j++)
    {
        PrintSiteBegin(2, psSiteAtt[j]);
        for (i = 0; i < lNbGateway; i++)
        {
            if ( !strcmp(psGatewayAtt[i].szSiteName, psSiteAtt[j].szName))
            {
                PrintGatewayBegin(3, psGatewayAtt[i]);
                PrintGatewayEnd(3);
                CDX_free(NULL,&(psGatewayAtt[i].sGtwModeList));
                CDX_free(NULL,&(psGatewayAtt[i].sMonitoringUnavailabilityList));
            }
        }    
         
        PrintSiteEnd(2);
    }
/* End OR7719 */ 

    PrintDomainEnd();
    PrintAdminServiceEnd();

    OS_mem_free((CSIVOID **)&psSiteAtt, &lReturnCode, &lReasonCode);
    OS_mem_free((CSIVOID **)&psGatewayAtt, &lReturnCode, &lReasonCode);
    return;
}

CSIBOOLEAN CheckMultimodeBackup(CSICHAR *szSiteString, CSICHAR *szGtwString)
{
    /* Check if we are running on the central backup gateway and
       requesting monitoring alerts for a mulimode Gw */
    CSIBOOLEAN isMultimodeBackup = CSIFALSE;
    CDXGTWATTINFO *psGatewayAtt;
    CSILONG lReturnCode;
    CSILONG lReasonCode;
    CSICHAR *szGtwMode;
    CSILONG lMonitoringFlags = 0L;

    if (CSISUCCESS != OS_mem_alloc(sizeof(*psGatewayAtt)
                                  ,(CSIVOID **)&psGatewayAtt
                                  ,&lReturnCode
                                  ,&lReasonCode
                                  )
       )
    {
        HandleError(COM_ERROR_OS
                   ,"OS_mem_alloc"
                   ,lReturnCode
                   ,lReasonCode
                   );
    }

    if (CSISUCCESS != CDX_getAttributesOfGateway(NULL
                                                ,getenv(CDX_C_ENV_SITE)
                                                ,getenv(CDX_C_ENV_GATEWAY)
                                                ,psGatewayAtt
                                                ,&lReturnCode
                                                ,&lReasonCode
                                                )
       )
    {
        HandleError(COM_ERROR_CDX
                   ,"CDX_getAttributesOfGateway"
                   ,lReturnCode
                   ,lReasonCode
                   );
    }
    szGtwMode = CDX_list_first(NULL, &(psGatewayAtt->sGtwModeList));
    if (strcasecmp(szGtwMode,CDX_C_BACKUP_STR) == 0)
    {
        /* We are running on the central backup Gw*/
        if (CSISUCCESS != CDX_getAttributesOfGateway(NULL
                                                    ,szSiteString
                                                    ,szGtwString
                                                    ,psGatewayAtt
                                                    ,&lReturnCode
                                                    ,&lReasonCode
                                                    )
           )
        {
            HandleError(COM_ERROR_CDX
                       ,"CDX_getAttributesOfGateway"
                       ,lReturnCode
                       ,lReasonCode
                       );
        }
        
        for (szGtwMode = CDX_list_first(NULL, &(psGatewayAtt->sGtwModeList))
            ;   NULL != szGtwMode
            ;szGtwMode = CDX_list_next(NULL, &(psGatewayAtt->sGtwModeList))
            )
        {
            if (!strcasecmp(szGtwMode, CDX_C_PRODUCTION_STR))
                lMonitoringFlags |= CDX_C_PRODUCTION;
            else if (!strcasecmp(szGtwMode, CDX_C_BACKUP_STR))
                lMonitoringFlags |= CDX_C_BACKUP;
        }
        isMultimodeBackup = ((lMonitoringFlags & CDX_C_PRODUCTION) && (lMonitoringFlags & CDX_C_BACKUP))
                            ? CSITRUE
                            : CSIFALSE;
    }

    OS_mem_free((CSIVOID **) &psGatewayAtt
                       ,&lReturnCode
                       ,&lReasonCode
                              );
							   
    return isMultimodeBackup;
}
