/****************************************************************************

         #####   #####  #     #       #  #####   #####    ###
        #     # #     # ##    #      #  #     # #     #    #
        #       #       # #   #     #   #       #          #
        #       #       #  #  #    #    #        #####     #
        #       #       #   # #   #     #             #    #
        #     # #     # #    ##  #      #     # #     #    #
         #####   #####  #     # #        #####   #####    ###

****************************************************************************/
/*
* @PROJECT  CCN/CSI
* @SYSTEM   APACHE
* @PACKAGE  MONMGMT 
*
* @FILE     monitor.yacc
*
* @AUTHOR   Stephane STEINIER
*
* @ABSTRACT
* Apache MONitoring ManaGeMenT (MONMGMT) : YACC file
*
* This file contains:
* the YACC directive to parse the Big Brother files
*
* @INCLUDE
*
*/
/*
* HISTORY
* ----------|-------|--------------------------------------------------------
*    Date   | Patch | Author & Description
* ----------|-------|--------------------------------------------------------
* 31-Oct-03 |OR05972| SST - Creation
* ----------|-------|--------------------------------------------------------
* 24-Nov-03 |OR06002| SST - Wrong alert when a syntax error is detected
* ----------|-------|--------------------------------------------------------
* 26-Nov-03 |OR06010| SST - Uniformisation of the error messages
* ----------|-------|--------------------------------------------------------
* 01-Dec-03 |OR06023| SST - XML escaping
* ----------|-------|--------------------------------------------------------
* 12-Dec-03 |OR06045| SST - OS_init call to initialise the OS context
* ----------|-------|--------------------------------------------------------
* 02-Mar-04 |OR06144| SST - RFA-112 implementation (https statistics)
* ----------|-------|--------------------------------------------------------
* 13-May-05 |OR06755| FER - RFA-160 implementation (monitoring NS50)
* ----------|-------|--------------------------------------------------------
* 23-Jun-05 |OR6905 | FER - RFA-160 Wrong colour in the portal when
*           |       |       there has been no mode change
* ----------|-------|--------------------------------------------------------
* 09-06-05  |  6740 | VVA - RFA-150 : Portage CCN/CSI sur Linux RedHAT3
* ----------|-------|--------------------------------------------------------
* 10-May-07 |OR08621| SST - Merge AiX -> Linux
* ----------|-------|--------------------------------------------------------
* 30-Jun-09 |OR10939| CLO - Output of tasks changed and cannot be parsed anymore
* ----------|-------|--------------------------------------------------------
* 04-Jan-10 |OR10575| CLO - Integrate Fwl/Ns50 in main monitoring
* ----------|-------|--------------------------------------------------------
* 18-Jan-10 |OR11554| TBA - Restore the print of the begining SERVICE tag
*           |       |       and the loop for the ALERTS
* ----------|-------|--------------------------------------------------------
*/

%{

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "gcn_uptyp.h"
#include "gcn_upcon.h"
#include "osx_upext.h"
#include "cdx_upcon.h"
#include "cdx_uptyp.h"
#include "cdx_upext.h"
#include "com_upext.h"

#include "monitor_prtyp.h"
#include "monitor_prcon.h"
#include "monitor_prext.h"

CSILONG iTask;
CSICHAR szDate[11];
CSICHAR szTime[9];
CSILONG iAlert;
CSICHAR* szHostName;
CSICHAR szObject[48];
CSICHAR szExecDate[11];
CSICHAR szExecTime[9];
CSICHAR szNS50AlertTime[22];
CSICHAR* szTimestamp;
ALERTTASK *psAlertTask = NULL;
CSIBOOLEAN NewFile;

/* not used but needed :-(( */
CSICHAR *szAction;
CSICHAR48 szWebDomain;
CSIBOOLEAN OSInit;
CSIBOOLEAN CDXBind;

int yylineno;
extern char yytext[];
extern int yylex(void);

static void yyerror(const char *msg)
{
    ALERTTASK *psCurrentTask = psAlertTask;
    CSIBOOLEAN found = CSIFALSE;
    CSICHAR1024 szErrorMessage;

    /* search for the highest alert */
    if (NULL != psCurrentTask)
    {   
        for (psCurrentTask = psAlertTask
            ;NULL   != psCurrentTask
            ;psCurrentTask = psCurrentTask->psNextTask
            )
        {    
            if (psCurrentTask->iTask == iTask)
            {
                found = CSITRUE;
                break;
            }
        }
    }

    if (!found)
        AddAlertTask(iTask
                    ,PURPLE
                    ,szExecDate
                    ,szExecTime
                    );         
    else
        psCurrentTask->iAlert = PURPLE;

    sprintf(szErrorMessage
           ,"%s in line %d at %s"
           ,msg
           ,yylineno
           ,yytext
           );

    AddTaskAlertLine(iTask
                    ,PURPLE
                    ,szHostName
                    ,szErrorMessage
                    );             
    return;
}



%}

%union 
{
    char *string;
}

%token <string> ALERT
%token <string> DATE
%token <string> TIME
%token <string> MESSAGE
%token <string> ERROR
%token <string> OBJECT

%%

file : first_line
     | first_line line

first_line : status date_time {
    AddAlertTask(iTask, iAlert, szDate, szTime);
}

line : line description_line
     | description_line

description_line : status error_message
     | status object message


date_time : date time  

status : ALERT  { iAlert = Alert_StringToInt($1); }
date : DATE     { strcpy(szDate, $1); }
time : TIME     { strcpy(szTime, $1); }
object : OBJECT { strcpy(szObject, $1); }

message : MESSAGE  { 
    AddTaskAlertLine(iTask, iAlert, szObject, $1);
}

error_message : ERROR  {
    snprintf(szObject, sizeof(szObject), "%s", "ERROR");
    AddTaskAlertLine(iTask, iAlert, szObject, $1);
}

%%

CSILONG Task_StringToInt(CSICHAR *szTask)
{
    if (!strcasecmp("rr", szTask))
        return RR;
    else if (!strcasecmp("async", szTask))
        return ASYNC;
    else if (!strcasecmp("http", szTask))
        return HTTP;
    else if (!strcasecmp("https", szTask))
        return HTTPS;
    else if (!strcasecmp("app", szTask))
        return APP;
    else if (!strcasecmp("que", szTask))
        return QUE;
    else if (!strcasecmp("usr", szTask))
        return USR;
    else if (!strcasecmp("fwl", szTask))
        return NS50;
    else
        return UNDEF;
}
    

CSICHAR *Task_IntToString(CSILONG iTask)
{
    if (iTask == RR)
        return "rr";
    else if (iTask == ASYNC)
        return "async";
    else if (iTask == HTTP)
        return "http";
    else if (iTask == HTTPS)
        return "https";
    else if (iTask == APP)
        return "app";
    else if (iTask == QUE)
        return "que";
    else if (iTask == NS50)
        return "fwl";
    else if (iTask == USR)
        return "usr";
    else
        return "undef";
}


CSILONG Alert_StringToInt(CSICHAR *szAlert)
{
    if (!strcasecmp(szAlert, "green"))
        return GREEN;
    else if (!strcasecmp(szAlert, "yellow"))
        return YELLOW;
    else if (!strcasecmp(szAlert, "red"))
        return RED;
    else if (!strcasecmp(szAlert, "purple"))
        return PURPLE;
    else
        return UNDEF;
}


CSICHAR *Alert_IntToString(CSILONG iAlert)
{
    if (iAlert == GREEN)
        return "green";
    else if (iAlert == YELLOW)
        return "yellow";
    else if (iAlert == RED)
        return "red";
    else
        return "purple";
}


CSIVOID AddAlertTask(CSILONG iTask
                    ,CSILONG iAlert
                    ,CSICHAR *szDate
                    ,CSICHAR *szTime
                    )
{
    ALERTTASK *psCurrentTask = NULL;

    psCurrentTask = malloc(sizeof(*psCurrentTask));
    if (NULL == psCurrentTask)
    {
    }

    psCurrentTask->iTask = iTask;
    psCurrentTask->iAlert = iAlert;
    strcpy(psCurrentTask->szDate, szDate);
    strcpy(psCurrentTask->szTime, szTime);
    psCurrentTask->psLines = NULL;
    psCurrentTask->psNextTask = psAlertTask;
    psAlertTask = psCurrentTask;

    return;
}


CSIVOID AddTaskAlertLine(CSILONG iTask
                        ,CSILONG iAlert
                        ,CSICHAR *szObject
                        ,CSICHAR *szMessage
                        )
{
    ALERTTASK *psCurrentTask = NULL;
    ALERTLINE *psCurrentLine = NULL;

    psCurrentTask = psAlertTask;
    while (   NULL != psCurrentTask
            && psCurrentTask->iTask != iTask
          )
    {
        psCurrentTask = psCurrentTask->psNextTask;
    }
    if (NULL == psCurrentTask)
    {
    }

    psCurrentLine = malloc(sizeof(*psCurrentLine));
    if (NULL == psCurrentLine)
    {
    }

    while (szMessage[0] == ' ') szMessage++;
    psCurrentLine->szMessage = malloc(strlen(szMessage) + 1);
    if (NULL == psCurrentLine->szMessage)
    {
    }
    
    psCurrentLine->iAlert = iAlert;
    strcpy(psCurrentLine->szObject, szObject);
    strcpy(psCurrentLine->szMessage, szMessage);
    psCurrentLine->psNextLine = psCurrentTask->psLines;
    psCurrentTask->psLines = psCurrentLine;

    return;
}


CSIVOID PrintXML(CSICHAR *szHostName
                ,CSICHAR *szDate
                ,CSICHAR *szTime
                ,CSICHAR *szTimestamp
                )
{
    struct tm sTimestamp;
    ALERTTASK *psCurrentTask = psAlertTask;
    CSILONG iHighestAlert = UNDEF;
    CSICHAR1024 szErrorMessage;


    /* search for the highest alert */
    if (NULL == psCurrentTask)
        iHighestAlert = PURPLE;
    else
    {
        for (psCurrentTask = psAlertTask
            ;   NULL   != psCurrentTask
             && PURPLE != iHighestAlert
            ;psCurrentTask = psCurrentTask->psNextTask
            )
        {
            if (psCurrentTask->iAlert > iHighestAlert)
                iHighestAlert = psCurrentTask->iAlert;
        }
    }

    PrintMonitorHostBegin(szHostName
                         ,Alert_IntToString(iHighestAlert)
                         ,szDate
                         ,szTime
                         );

    /* printing all alert */
    for (psCurrentTask = psAlertTask
        ;NULL != psCurrentTask
        ;psCurrentTask = psCurrentTask->psNextTask
        )
    {

        ALERTLINE *psCurrentLine;

        PrintMonitorServiceBegin(Task_IntToString(psCurrentTask->iTask)
                                ,Alert_IntToString(psCurrentTask->iAlert)
                                ,psCurrentTask->szDate
                                ,psCurrentTask->szTime
                                );
        for (psCurrentLine = psCurrentTask->psLines
            ;NULL != psCurrentLine
            ;psCurrentLine = psCurrentLine->psNextLine
            )
        {
            PrintMonitorAlert(psCurrentLine->szObject
                             ,Alert_IntToString(psCurrentLine->iAlert)
                             ,psCurrentLine->szObject
                             ,psCurrentLine->szMessage
                            );
        }
        PrintMonitorServiceEnd();
    }
   
    PrintMonitorHostEnd(); 

    return;
}


int main(int argc, char *argv[])
{
    CSICHAR *szTask = (CSICHAR *) 0;
    extern FILE *yyin;
    time_t tExecDateTime;
    struct tm *psExecDateTime;
    CSILONG lReturnCode, lReasonCode;

    if (CSISUCCESS != OS_init("MONMGMT"
                             ,&lReturnCode
                             ,&lReasonCode
                             )
       )
    {
        return 1;
    }
    tExecDateTime =  time(NULL);
    psExecDateTime = gmtime(&tExecDateTime);
    strftime(szExecDate
            ,sizeof(szExecDate)
            ,"%d/%m/%Y"
            ,psExecDateTime
            );
    strftime(szExecTime
            ,sizeof(szExecTime)
            ,"%H:%M:%S"
            ,psExecDateTime
            );

    if (argc > 1)
    {
        int i;

        for (i = 1; i < argc; i++)
        {
            FILE *pf;
                    
            szHostName = strdup(argv[i]);
            if (NULL == szHostName)
                return 1;
            if (NULL != strrchr(szHostName, '/'))
                szHostName = strrchr(szHostName, '/') + 1;
            szTask = strrchr(szHostName, '.');
            if (NULL == szTask)
               continue;
            szTask[0] = '\0';
            szTask++;
            iTask = Task_StringToInt(szTask);
            if (iTask == UNDEF)
                continue;
             
            if (!strcmp(szTask, "mode"))
            {
                szTimestamp = strdup(argv[i+1]);
                argc = i;
            }
            pf = fopen(argv[i], "r");
            if (NULL != pf)
            {
                NewFile = CSITRUE;
                yyin = pf;
                yyparse();
                fclose(pf);
            }
            else
            {
                CSICHAR256 szErrorMessage;
                if (!strcmp(szTask, "mode"))
                {
                    sprintf(szErrorMessage
                           ,"No mode change logged for device %s\n"
                           ,szHostName
                           );
    
                    AddAlertTask(iTask
                                ,GREEN
                                ,szExecDate
                                ,szExecTime
                                );
                    AddTaskAlertLine(iTask
                                    ,GREEN
                                    ,szHostName
                                    ,szErrorMessage
                                    );
                }
                else
                {   
                    sprintf(szErrorMessage
                           ,"Problem while opening Big Brother file %s\n"
                           ,argv[i]
                           );
                
                    AddAlertTask(iTask
                                ,PURPLE
                                ,szExecDate
                                ,szExecTime
                                );
                    AddTaskAlertLine(iTask
                                    ,PURPLE
                                    ,szHostName
                                    ,szErrorMessage
                                    );
                }
            }
        }
        if ((CSICHAR *) 0 != szTask && !strcmp(szTask, "mode"))
        {   
            PrintXML(szHostName, szExecDate, szExecTime, szTimestamp);
        }   
        else
        {
            PrintXML(szHostName, szExecDate, szExecTime, NULL);
        }
    }

    OS_term(&lReturnCode, &lReasonCode);

    return 0;
}

